@@ -3,8 +3,8 @@ ClusterRegistry <- local({
   last <- NULL
   cluster <- NULL
 
-  function(action = c("get", "start", "stop"), workers = NULL, ...) {
-    action <- match.arg(action)
+  function(action = c("get", "start", "stop"), workers = NULL, makeCluster = .makeCluster, ...) {
+    action <- match.arg(action, choices = c("get", "start", "stop"))
 
     if (is.null(workers)) {
     } else if (is.numeric(workers)) {
@@ -18,7 +18,7 @@ ClusterRegistry <- local({
     }
 
     if (length(cluster) == 0L && action != "stop") {
-      cluster <<- .makeCluster(workers, ...)
+      cluster <<- makeCluster(workers, ...)
       last <<- workers
     }
 
@@ -28,7 +28,7 @@ ClusterRegistry <- local({
       ## Already setup?
       if (!identical(workers, last)) {
         ClusterRegistry(action = "stop")
-        cluster <<- .makeCluster(workers, ...)
+        cluster <<- makeCluster(workers, ...)
         last <<- workers
       }
     } else if (action == "stop") {
@@ -44,5 +44,25 @@ ClusterRegistry <- local({
 
 .makeCluster <- function(workers, ...) {
   if (length(workers) == 0L) return(NULL)
-  makeClusterPSOCK(workers, ...)
+  cl <- makeClusterPSOCK(workers, ...)
+  cl <- addCovrLibPath(cl)
+  cl
 } ## .makeCluster()
+
+
+#' @importFrom parallel clusterCall
+addCovrLibPath <- function(cl) {
+  if (!is.element("covr", loadedNamespaces())) return(cl)
+  debug <- getOption("future.debug", FALSE)
+  
+  ## WORKAROUND: When running covr::package_coverage(), the
+  ## package being tested may actually not be installed in
+  ## library path used by covr.  We here add that path iff
+  ## covr is being used. /HB 2016-01-15
+  if (debug) mdebug("covr::package_coverage() workaround ...")
+  libPath <- .libPaths()[1]
+  clusterCall(cl, fun = function() .libPaths(c(libPath, .libPaths())))
+  if (debug) mdebug("covr::package_coverage() workaround ... DONE")
+
+  cl
+}
\ No newline at end of file
