@@ -4,11 +4,35 @@
 #' which means that its _value is computed and resolved in
 #' parallel in another \R session_.
 #'
+#' @details
+#' This function is _not_ meant to be called directly.  Instead, the
+#' typical usages are:
+#'
+#' ```r
+#' # Evaluate futures in parallel on the local machine via as many background
+#' # processes as available to the current R process
+#' plan(multisession)
+#'
+#' # Evaluate futures in parallel on the local machine via two background
+#' # processes
+#' plan(multisession, workers = 2)
+#' ```
+#'
 #' @inheritParams multiprocess
 #' @inheritParams cluster
 #' @inheritParams Future-class
 #' @inheritParams future
 #'
+#' @param rscript_libs A character vector of \R package library folders that
+#' the workers should use.  The default is `.libPaths()` so that multisession
+#' workers inherits the same library path as the main \R session.
+#' To avoid this, use `plan(multisession, ..., rscript_libs = NULL)`.
+#' _Important: Note that the library path is set on the workers when they are
+#' created, i.e. when `plan(multisession)` is called.  Any changes to
+#' `.libPaths()` in the main R session after the workers have been created
+#' will have no effect._
+#' This is passed down as-is to [parallelly::makeClusterPSOCK()].
+#'
 #' @return A \link{MultisessionFuture}.
 #' If `workers == 1`, then all processing using done in the
 #' current/main \R session and we therefore fall back to using
@@ -23,33 +47,21 @@
 #' The background \R sessions (the "workers") are created using
 #' [makeClusterPSOCK()].
 #' 
-#' The `multisession()` function will block if all available
-#' \R session are occupied
-#' and will be unblocked as soon as one of the already running
-#' multisession futures is resolved.  For the total number of
+#' For the total number of
 #' \R sessions available including the current/main \R process, see
-#' [availableCores()].
+#' [parallelly::availableCores()].
 #'
 #' A multisession future is a special type of cluster future.
 #'
-#' The preferred way to create an multisession future is not to call
-#' this function directly, but to register it via
-#' \code{\link{plan}(multisession)} such that it becomes the default
-#' mechanism for all futures.  After this [future()]
-#' and \code{\link{\%<-\%}} will create _multisession futures_.
-#'
 #' @seealso
 #' For processing in multiple forked \R sessions, see
 #' [multicore] futures.
-#' For multicore processing with fallback to multisession where
-#' the former is not supported, see [multiprocess] futures.
 #'
-#' Use [availableCores()] to see the total number of
+#' Use [parallelly::availableCores()] to see the total number of
 #' cores that are available for the current \R session.
 #'
 #' @export
-multisession <- function(expr, envir = parent.frame(), substitute = TRUE, lazy = FALSE, seed = NULL, globals = TRUE, persistent = FALSE, workers = availableCores(), gc = FALSE, earlySignal = FALSE, label = NULL, ...) {
-  if (substitute) expr <- substitute(expr)
+multisession <- function(..., workers = availableCores(), lazy = FALSE, rscript_libs = .libPaths(), envir = parent.frame()) {
   if (is.function(workers)) workers <- workers()
   workers <- as.integer(workers)
   stop_if_not(length(workers) == 1, is.finite(workers), workers >= 1)
@@ -57,14 +69,13 @@ multisession <- function(expr, envir = parent.frame(), substitute = TRUE, lazy =
   ## Fall back to lazy sequential futures if only a single R session can be used,
   ## that is, then use the current main R process.
   if (workers == 1L) {
-    return(sequential(expr, envir = envir, substitute = FALSE, lazy = TRUE, seed = seed, globals = globals, local = TRUE, label = label, ...))
+    return(sequential(..., lazy = TRUE, envir = envir))
   }
 
-  workers <- ClusterRegistry("start", workers = workers)
-
-  future <- MultisessionFuture(expr = expr, envir = envir, substitute = FALSE, lazy = lazy, seed = seed, globals = globals, persistent = persistent, workers = workers, gc = gc, earlySignal = earlySignal, label = label, ...)
+  future <- MultisessionFuture(..., workers = workers, lazy = lazy, rscript_libs = rscript_libs, envir = envir)
   if (!future$lazy) future <- run(future)
   invisible(future)
 }
 class(multisession) <- c("multisession", "cluster", "multiprocess", "future", "function")
 attr(multisession, "init") <- TRUE
+attr(multisession, "untweakable") <- c("persistent")
