@@ -55,7 +55,7 @@
 #'    _forked_ \R processes running in the background on
 #'    the same machine.  Not supported on Windows.
 #'  }
-#'  \item{[`multiprocess`]:}{
+#'  \item{[`multiprocess`]:}{(DEPRECATED)
 #'    If multicore evaluation is supported, that will be used,
 #'    otherwise multisession evaluation will be used.
 #'  }
@@ -93,12 +93,21 @@
 #' If you think it is necessary to modify the future strategy within a
 #' function, then make sure to undo the changes when exiting the function.
 #' This can be done using:
+#'
 #' \preformatted{
 #'   oplan <- plan(new_set_of_strategies)
 #'   on.exit(plan(oplan), add = TRUE)
 #'   [...]
 #' }
 #'
+#' This is important because the end-user might have already set the future
+#' strategy elsewhere for other purposes and will most likely not known that
+#' calling your function will break their setup.
+#' _Remember, your package and its functions might be used in a greater
+#' context where multiple packages and functions are involved and those might
+#' also rely on the future framework, so it is important to avoid stepping on 
+#' others' toes._
+#'
 #' @section Using plan() in scripts and vignettes:
 #' When writing scripts or vignettes that uses futures, try to place any
 #' call to `plan()` as far up (as early on) in the code as possible.  
@@ -124,6 +133,81 @@ plan <- local({
   ## Stack of type of futures to use
   stack <- defaultStack
 
+  assert_no_disallowed_strategies <- function(stack) {
+    noplans <- getOption("future.plan.disallow")
+    if (length(noplans) == 0L) return()
+
+    for (kk in seq_along(stack)) {
+      evaluator <- stack[[kk]]
+      if (!inherits(evaluator, noplans)) next
+      clazz <- class(evaluator)[1]
+      if (!clazz %in% noplans) next  ## <== sic!
+
+      stop(FutureError(sprintf("Can not use %s in the future plan because it is on the list of future strategies that are not allow per option 'future.plan.disallow': %s", sQuote(clazz), paste(sQuote(noplans), collapse = ", "))))
+    }
+  }
+
+  warn_about_multiprocess <- local({
+    .warn <- TRUE
+
+    is_multiprocess <- function(evaluator) {
+      if (!inherits(evaluator, "multiprocess")) return(FALSE)
+      ## NOTE: Yes, we are indeed inspecting the 'class' attribute itself
+      class <- class(evaluator)
+      if (class[1] == "multiprocess") return(TRUE)
+      if (length(class) == 1L) return(FALSE)
+      if (class[1] == "tweaked" && class[2] == "multiprocess") return(TRUE)
+      FALSE
+    }
+
+    function(stack) {
+      if (!.warn) return()
+
+      ## Is deprecated 'multiprocess' used?    
+      for (kk in seq_along(stack)) {
+        if (is_multiprocess(stack[[kk]])) {
+          ignore <- getOption("future.deprecated.ignore")
+          if (!is.element("multiprocess", ignore)) {
+            ## Warn only once
+            .warn <<- FALSE
+            .Deprecated(msg = sprintf("Strategy 'multiprocess' is deprecated in future (>= 1.20.0). Instead, explicitly specify either 'multisession' or 'multicore'. In the current R session, 'multiprocess' equals '%s'.", if (supportsMulticore()) "multicore" else "multisession"), package = .packageName)
+          }
+
+          break
+        }
+      }
+    }
+  })
+
+  warn_about_multicore <- local({
+    .warn <- TRUE
+
+    is_multicore <- function(evaluator) {
+      if (!inherits(evaluator, "multicore")) return(FALSE)
+      ## NOTE: Yes, we are indeed inspecting the 'class' attribute itself
+      class <- class(evaluator)
+      if (class[1] == "multicore") return(TRUE)
+      if (length(class) == 1L) return(FALSE)
+      if (class[1] == "tweaked" && class[2] == "multicore") return(TRUE)
+      FALSE
+    }
+
+    function(stack) {
+      if (!.warn) return()
+
+      ## Is 'multicore' used despite not being supported on the current
+      ## platform?    
+      for (kk in seq_along(stack)) {
+        if (is_multicore(stack[[kk]])) {
+          supportsMulticore(warn = TRUE)
+          ## Warn only once, if at all
+          .warn <<- FALSE
+          break
+        }
+      }
+    }
+  })
+
   plan_cleanup <- function() {
     ClusterRegistry(action = "stop")
   }
@@ -202,7 +286,12 @@ plan <- local({
       mdebug("plan(): Setting new future strategy stack:")
       mprint(newStack)
     }
-    
+
+    assert_no_disallowed_strategies(newStack)
+
+    warn_about_multiprocess(newStack)
+    warn_about_multicore(newStack)
+
     stack <<- newStack
 
     ## Stop any (implicitly started) clusters?
@@ -237,6 +326,7 @@ plan <- local({
       if (!inherits(strategy, "FutureStrategy")) {
         class(strategy) <- c("FutureStrategy", class(strategy))
       }
+      stop_if_not(is.function(strategy))
       return(strategy)
     } else if (identical(strategy, "default")) {
       strategy <- getOption("future.plan", sequential)
@@ -500,6 +590,6 @@ resetWorkers.multicore <- function(x, ...) {
   if (usedCores() == 0L) return(invisible(x))
   reg <- sprintf("multicore-%s", session_uuid())
   FutureRegistry(reg, action = "collect-all", earlySignal = FALSE)
-  stopifnot(usedCores() == 0L)
+  stop_if_not(usedCores() == 0L)
 }
 
